=== modified file 'bzrlib/config.py'
--- bzrlib/config.py	2006-09-18 02:59:05 +0000
+++ bzrlib/config.py	2006-10-02 13:36:02 +0000
@@ -426,6 +426,13 @@
             # if section is longer, no match.
             if len(section_names) > len(location_names):
                 continue
+            # if path is longer, and recurse is not true, no match
+            if len(section_names) < len(location_names):
+                try:
+                    if not self._get_parser()[section].as_bool('recurse'):
+                        continue
+                except KeyError:
+                    pass
             matches.append((len(section_names), section,
                             '/'.join(location_names[len(section_names):])))
         matches.sort(reverse=True)

=== modified file 'bzrlib/tests/test_config.py'
--- bzrlib/tests/test_config.py	2006-09-18 02:59:05 +0000
+++ bzrlib/tests/test_config.py	2006-10-02 13:36:02 +0000
@@ -67,6 +67,9 @@
                         "[http://www.example.com/ignoreparent]\n"
                         "# different project: ignore parent dir config\n"
                         "ignore_parents=true\n"
+                        "[http://www.example.com/norecurse]\n"
+                        "# configuration items that only apply to this dir\n"
+                        "recurse=false\n"
                         "[/b/]\n"
                         "check_signatures=require\n"
                         "# test trailing / matching with no children\n"
@@ -552,17 +555,29 @@
         self.assertEqual([('http://www.example.com', 'com')],
                          self.my_location_config._get_matching_sections())
 
-    def test__get_matching_sections_subdir_matches(self):
+    def test__get_matching_sections_ignoreparent(self):
         self.get_branch_config('http://www.example.com/ignoreparent')
         self.assertEqual([('http://www.example.com/ignoreparent', '')],
                          self.my_location_config._get_matching_sections())
 
-    def test__get_matching_sections_subdir_nonrecursive(self):
+    def test__get_matching_sections_ignoreparent_subdir(self):
         self.get_branch_config(
             'http://www.example.com/ignoreparent/childbranch')
         self.assertEqual([('http://www.example.com/ignoreparent', 'childbranch')],
                          self.my_location_config._get_matching_sections())
 
+    def test__get_matching_sections_norecurse(self):
+        self.get_branch_config('http://www.example.com/norecurse')
+        self.assertEqual([('http://www.example.com/norecurse', ''),
+                          ('http://www.example.com', 'norecurse')],
+                         self.my_location_config._get_matching_sections())
+
+    def test__get_matching_sections_norecurse_subdir(self):
+        self.get_branch_config(
+            'http://www.example.com/norecurse/childbranch')
+        self.assertEqual([('http://www.example.com', 'norecurse/childbranch')],
+                         self.my_location_config._get_matching_sections())
+
     def test__get_matching_sections_subdir_trailing_slash(self):
         self.get_branch_config('/b')
         self.assertEqual([('/b/', '')],

